#include "midi_handler.h"
#include "config.h"

// Statische Callback-Pointer initialisieren
void (*MIDIHandler::onNoteOnCallback)(uint8_t note, uint8_t velocity) = nullptr;
void (*MIDIHandler::onNoteOffCallback)(uint8_t note, uint8_t velocity) = nullptr;

void MIDIHandler::init() {
  // MIDI USB wird automatisch initialisiert
  Serial.println("MIDI Handler initialisiert - USB MIDI bereit");
}

void MIDIHandler::noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
}

void MIDIHandler::noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
}

void MIDIHandler::controlChange(byte channel, byte control, byte value) {
  midiEventPacket_t event = {0x0B, 0xB0 | channel, control, value};
  MidiUSB.sendMIDI(event);
}

void MIDIHandler::sendNoteOn(uint8_t note, uint8_t velocity) {
  noteOn(MIDI_CHANNEL - 1, note, velocity); // MIDI-Kanäle sind 0-basiert
  MidiUSB.flush();
  
  Serial.print("MIDI Note-On gesendet: Note ");
  Serial.print(note);
  Serial.print(", Velocity ");
  Serial.println(velocity);
}

void MIDIHandler::sendNoteOff(uint8_t note, uint8_t velocity) {
  noteOff(MIDI_CHANNEL - 1, note, velocity); // MIDI-Kanäle sind 0-basiert
  MidiUSB.flush();
  
  Serial.print("MIDI Note-Off gesendet: Note ");
  Serial.print(note);
  Serial.print(", Velocity ");
  Serial.println(velocity);
}

void MIDIHandler::sendControlChange(uint8_t controller, uint8_t value) {
  controlChange(MIDI_CHANNEL - 1, controller, value); // MIDI-Kanäle sind 0-basiert
  MidiUSB.flush();
  
  Serial.print("MIDI CC gesendet: Controller ");
  Serial.print(controller);
  Serial.print(", Wert ");
  Serial.println(value);
}

void MIDIHandler::processIncomingMIDI() {
  midiEventPacket_t rx;
  
  do {
    rx = MidiUSB.read();
    
    if (rx.header != 0) {
      // Extrahiere MIDI-Daten
      uint8_t messageType = rx.byte1 & 0xF0;
      uint8_t channel = rx.byte1 & 0x0F;
      uint8_t data1 = rx.byte2;
      uint8_t data2 = rx.byte3;
      
      // Prüfe auf den konfigurierten MIDI-Kanal
      if(channel == (MIDI_CHANNEL - 1)) {
        switch(messageType) {
          case 0x90: // Note-On
            if(data2 > 0) { // Velocity > 0 = echtes Note-On
              Serial.print("MIDI Note-On empfangen: Note ");
              Serial.print(data1);
              Serial.print(", Velocity ");
              Serial.println(data2);
              
              if(onNoteOnCallback != nullptr) {
                onNoteOnCallback(data1, data2);
              }
            } else { // Velocity = 0 = Note-Off
              Serial.print("MIDI Note-Off empfangen (via Note-On): Note ");
              Serial.println(data1);
              
              if(onNoteOffCallback != nullptr) {
                onNoteOffCallback(data1, 0);
              }
            }
            break;
            
          case 0x80: // Note-Off
            Serial.print("MIDI Note-Off empfangen: Note ");
            Serial.println(data1);
            
            if(onNoteOffCallback != nullptr) {
              onNoteOffCallback(data1, data2);
            }
            break;
        }
      }
    }
  } while (rx.header != 0);
}
